	!
	function integer neil_CreLnm (string	logical_name$,	&
						equiv_name$,	&
						table_name$	)
	!==============================================================================================================
	! Title  : neil_crelnm.fun
	! Author : Neil S. Rieck (http://www3.sympatico.ca/n.rieck/)
	! Created: 1991.09.11
	! Purpose: an external function to create SUPERVISOR MODE logical names
	! History:
	! ver who when   what
	! --- --- ------ ----------------------------------------------------------------------------------------------
	! 100 NSR 910911 1. original work
	! 101 NSR 980625 1. cleaned up and optimzed
	!     NSR 000504 2. replaced external references with calls to STARLET
	! 102 NSR 040513 1. added access mode support
	!     NSR 040520 2. now only use executive-mode
	!		 3. now call neil_C_CRELNM4 via sys$cmexec
	! 103 NSR 040521 1. now call neil_C_CRELNM5 via sys$cmexec
	!--------------------------------------------------------------------------------------------------------------
	! Notes:
	! 1. this function requires an additional 'c' function to be linked
	! 2. by default, SYS$CRELNM can only create USER MODE logicals only. To create a name in SUPERVISOR MODE you
	!    must either:
	!	a. first switch to executive mode via sys$cmexec then call sys$crelnm
	!	b. use LIB$SET_LOGICAL if you have an attached CLI.
	! 3. you need CMEXEC priv to call sys$cmexec
	! 4. you need SYSNAM priv to write to SUPERVISOR MODE logicals in table LNM$SYSTEM
	! 5. please include the next line near the top of your source program
	!	external integer function neil_CreLnm (string, string, string)
	! 6. please include the next 2 lines near the bottom of of your source program (after 'END' of the main module)
	!	%include "[.fun]neil_crelnm.fun"
	!!	! function integer neil_CreLnm( logical_name$, equiv_name$, table_name$)
	!==============================================================================================================
	option	type=explicit							! cuz tricks are for kids...
	!
	!	<<< declare constants >>>
	!
	%include "starlet" %from %library "sys$library:basic$starlet"
	%include "$ssdef"  %from %library "sys$library:basic$starlet"
	%include "$lnmdef" %from %library "sys$library:basic$starlet"
	%include "$psldef" %from %library "sys$library:basic$starlet"
	!
	!	<<< declare variables >>>
	!
	declare 	long	rc%				,		! return code			&
			long	remote_rc%					!
	declare 	byte	accessmode					!
	!
	!	<<< declare types >>>
	!
	record ItemRec
		word Buf_Len							! buffer size (in bytes)
		word Code							! desired operation
		long Buf_Addr							! buffer address
		long Count_Addr							! addr of bytes returned
	end record
	!
	record	LogRec								! define a new data type
		ItemRec	ItemVar(0)						! 1 item (0)
		long	list_terminator
	end record LogRec
	!
	!	<<< declare variables using new type(s) >>>
	!
	declare	LogRec	TrnBuf							!
	declare long  ArgList(5)						! 5+1 parameters
	!
	logical_name$ = edit$(logical_name$,32+2)
	equiv_name$ = edit$(equiv_name$,32+2)
	!
	!==================================================
	! prep for call (fill in table)
	!==================================================
	!
	declare string	temp$	,&
		long	temp%
	declare long constant MapSize% = 31%					!
	map (CreLnmMap) string equiv_map = MapSize%				!
	!
	temp$ = edit$(equiv_name$,   32+4+2)					! clean up equiv name
	temp% = len(temp$)							! determine the string length
	equiv_map = temp$							! copy dynamic -> static
	!
	TrnBuf::ItemVar(0%)::buf_len	= temp%					! length
	TrnBuf::ItemVar(0%)::code	= lnm$_String				! desired op code
	TrnBuf::ItemVar(0%)::buf_addr	= loc(Equiv_Map)			! address
	TrnBuf::ItemVar(0%)::Count_Addr	= 0%					! don't care
	TrnBuf::List_Terminator		= 0%					! that's all...
	!
	accessmode	= PSL$C_SUPER						! we only want supervisor mode
	if pos( edit$(table_name$,32+2), "LNM$SYSTEM", 1) >0			! if trying to update a system table
	then									!
	    external long neil_c_CreLnm5					! this is a 'c' version of our function (but
										! ...don't declare the parameter list)
	    ArgList(0)	= 5							! we have 5 parameters after this one
	    ArgList(1)	= loc(table_name$)					!	dynamic string
	    ArgList(2)	= loc(logical_name$)					!	dynamic string
	    ArgList(3)	= loc(TrnBuf)						!	list
	    ArgList(4)	= loc(accessmode)					!	byte
	    ArgList(5)	= loc(remote_rc%)					!	long
	    rc% = sys$cmexec( 							! switch to executive mode			&
		neil_c_CreLnm5			,				! ...while running this module			&
		ArgList(0)				)			! ...with this argument list
	    if (rc% and 7% ) = 1 then						! if all went well with the system call
		rc% = remote_rc%						! ...then copy the rc passed back from the function
		goto test_rc							! ...and jump
	    else								! else
		print ""							! ...assum a priv problem
		print "-e-SYSERR_a_" + str$(rc%) + bel + " In 'neil_CreLnm'"	! ...print a message and exit
		sleep 2
		goto exit_rc
	    end if
	end if
	!
	!	only used when...
	!	1. not updating a SYSTEM table
	!
	rc% = sys$crelnm(					,		! attributes	&
				edit$( table_name$	,32+2)	,		! table name	&
				edit$( logical_name$	,32+2)	,		! logical name	&
				accessmode			,		! access mode	&
				TrnBuf				)		! item list
	!
	test_rc:
	if (rc% and 7%) <> 1% then
	    print "-e-SYSERR_b_" + str$(rc%) + bel + " In 'neil_CreLnm'"
	    sleep 2
	end if
	!
	!	pass return code back to caller
	!
	exit_rc:
	neil_CreLnm = rc%
	!
	end function
